import React, { useEffect, useState } from 'react';
import {Tabs} from 'antd';
import {Login} from './auth.login';
import { ResetPassword} from './auth.resetPassword';
import {LoginScan} from './auth.loginScan';
import {YAuthProps} from './auth.interface';
import {useModel} from './useModel';
import {LoginStyle, FormStyle, LoginLogoStyle, FormBoxStyle} from './styles';
import loginLogo from './login-logo.png';
import {useSystemContext} from '@/system/system.context';

let videoPath = "https://yun.ywwl.com/static/media/light.781d0d4a.mp4";
const { TabPane } = Tabs;

export const YAuth : React.FC<YAuthProps> = ({onSubmit}) => {
  const {store} = useSystemContext();
  const [state] = store();
  const { switchAuthType } = useModel();
  const switchType = (key: string) => {
    if(key === 'loginByPassword'){
      switchAuthType('login');
    }
  }

  const loadVideo = () => {
    const currentHours = new Date().getHours()
    if (currentHours < 7 || currentHours >= 19) {
      return videoPath = 'https://yun.ywwl.com/static/media/dark.ca3b5ddd.mp4'
    }
  }

  useEffect(() => {
    loadVideo()
  },[]);

  const type = state.authType;

  return (
    <LoginStyle>
      <video
        autoPlay={true}
        loop
        preload="true"
        muted
        playsInline={true}
        src={videoPath}
        className="video">
        <source src={videoPath} type="video/mp4"></source>
        <source src={videoPath} type="audio/ogg"></source>
        <source src={videoPath} type="audio/mpeg"></source>
      </video>
      <FormStyle>
        <FormBoxStyle>
          <LoginLogoStyle src={loginLogo} />
          {
            type === "login" && (
              <Tabs defaultActiveKey="loginByPassword" onChange={switchType}>
                <TabPane tab="账号密码登录" key="loginByPassword">
                  <Login onSubmit={onSubmit} />
                </TabPane>
                <TabPane tab="扫码登录" key="loginByScan">
                  <LoginScan />
                </TabPane>
              </Tabs>
            )
          }
          {
            type === "reset password" && <ResetPassword onSubmit={onSubmit} />
          }
        </FormBoxStyle>
      </FormStyle>
    </LoginStyle>
  )
}
